@@ -50,10 +50,14 @@ extern gint src_speech_mode;
 gboolean src_mouse_take;
 gboolean src_mouse_click;
 gboolean src_enable_format_string = FALSE; 
+gint src_caret_position;
 
 gchar *braille_device = NULL;
 gint   braille_port;
 
+extern gchar *scroll;
+extern gint old_offset;
+
 extern gboolean panning;
 
 
@@ -90,7 +94,7 @@ src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
 
     srl_assert (src_use_braille || src_use_braille_monitor);
 
-	
+	src_caret_position = -1;
     src_enable_format_string = FALSE; 
     txt = src_presentation_sro_get_pres_for_device (obj, reason, "braille");
     if (txt)
@@ -104,22 +108,27 @@ src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
 	gchar *braille_translation_table = NULL;
 	gint  braille_offset;
 	gchar offset[5];
+	gchar *braille_scroll = NULL;
 
 	braille_style = src_braille_get_style ();
 	braille_cursor_style = src_braille_get_cursor_style ();
 	braille_translation_table = src_braille_get_translation_table ();
 	braille_offset = src_braille_get_offset();
+	braille_scroll = g_strdup (scroll);
 	
 	sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL)
+	if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL || role == SR_ROLE_COMBO_BOX)
 	{
 	    SRLong offset;
 	    SRTextAttribute *attr;
 	    gchar status_[5] = "    ";
 
 	    if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
-		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset, braille_cursor_style);
-	    
+		{
+			if (src_caret_position == -1)
+				src_caret_position = 0;
+		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset+src_caret_position, braille_cursor_style);
+	    }
 	    if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
 				    &attr, SR_INDEX_CONTAINER))
 	    {
@@ -163,10 +172,25 @@ src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
 			cursor ? cursor : "",
 			">",
 			txt,
-			"</BRLDISP>",
-			status ? status : "",
-			"</BRLOUT>",
 			NULL);
+	if (braille_scroll && braille_scroll[0])
+	{
+	    tmp = g_strconcat (tmp, 
+			       "<SCROLL mode=\"", braille_scroll, "\"></SCROLL>", 
+			       "</BRLDISP>",
+			       status ? status : "",
+	                       "</BRLOUT>", 
+			       NULL);		       
+	    g_free (braille_scroll);
+	}		
+	else
+	{
+	    tmp = g_strconcat (tmp, 
+			       "</BRLDISP>",
+			       status ? status : "",
+	                       "</BRLOUT>", 
+			       NULL);
+	}		
 	/*fprintf (stderr, "\n%s", tmp);*/
 	if (src_use_braille)
 	    src_braille_send (tmp);
@@ -230,14 +254,23 @@ src_present_real_sro_for_speech (SRObject *obj, gchar *reason)
 	if (txt && txt[0])
 	{
 	    SRObjectRoles role;
+	    SRState state;
 	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if ( role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
-		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
-	    else if ((role == SR_ROLE_TABLE || role == SR_ROLE_TREE_TABLE) &&
-		strcmp (reason , "focus:") == 0)
-		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
+	    sro_get_state (obj, &state, SR_INDEX_CONTAINER);
+	    if (state & SR_STATE_FOCUSED || strcmp (reason, "present") == 0)
+	    {
+		if ( role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
+		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
+		else if ((role == SR_ROLE_TABLE || role == SR_ROLE_TREE_TABLE) &&
+			strcmp (reason , "focus:") == 0)
+		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, FALSE);
+		else
+		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
+	    }
 	    else
-		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
+	    {
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
+	    }
 	}
 	g_free (txt);
     }
@@ -274,7 +307,19 @@ src_present_real_sro (SRObject *obj)
 	return;
     
     if (old != obj)
+    {	
 	src_braille_set_offset (0);
+	old_offset = 0;
+	if (scroll)
+	{
+	    g_free (scroll);
+	    scroll = NULL;
+        }
+     }
+    else
+    {
+	src_braille_set_offset (old_offset);
+    }
     old = obj;
 
     if (!src_use_braille && !src_use_speech && !src_use_magnifier && !src_use_braille_monitor)
@@ -700,6 +745,7 @@ src_process_config_changed_for_srcore (SRConfigStructure *config)
 		src_use_braille = *((gboolean*)config->newvalue);
 		if (src_use_braille)
 		{
+		    src_braille_get_defaults ();
 		    src_use_braille = src_braille_init ();
 		}
 		else
@@ -1119,37 +1165,37 @@ src_process_config_changed_for_magnifier (SRConfigStructure *config)
     }
     else if (strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_ZOOM_FACTOR_X) == 0)
     {
-	gint zoom_factor;
+	gdouble zoom_factor;
 	gboolean rv;
 	
-	sru_assert (config->type == CFGT_INT && config->newvalue);
+	sru_assert (config->type == CFGT_FLOAT && config->newvalue);
 	
-	zoom_factor = *((gint*)config->newvalue);
+	zoom_factor = *((gdouble*)config->newvalue);
 	rv = src_magnifier_set_zoom_factor_x (zoom_factor);
 	
 	if (rv && src_use_speech)
 	{
 	    gchar *tmp;
-	    tmp = g_strdup_printf ("%s %d", _("zoom factor x"), zoom_factor);
+	    tmp = g_strdup_printf ("%s %f", _("zoom factor x"), zoom_factor);
 	    src_say_message (tmp);
 	    g_free (tmp);
 	}
     }
     else if (strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_ZOOM_FACTOR_Y) == 0)
     {
-	gint zoom_factor;
+	gdouble zoom_factor;
 	gboolean rv;
-
-	sru_assert (config->type == CFGT_INT && config->newvalue); 
-
-	zoom_factor = *((gint*)config->newvalue);
+	
+	sru_assert (config->type == CFGT_FLOAT && config->newvalue);
+	
+	zoom_factor = *((gdouble*)config->newvalue);
 	rv = src_magnifier_set_zoom_factor_y (zoom_factor);
 	
 	if (rv && src_use_speech)
 	{
-    	    gchar *tmp;
-	    tmp = g_strdup_printf ("%s %d", _("zoom factor y"), zoom_factor);
-    	    src_say_message (tmp);
+	    gchar *tmp;
+	    tmp = g_strdup_printf ("%s %f", _("zoom factor y"), zoom_factor);
+	    src_say_message (tmp);
 	    g_free (tmp);
 	}
     }
@@ -1340,7 +1386,7 @@ src_process_config_changed (SREvent *event,
 		printf ("%i\n", *((gint*)config->newvalue));
 		break;
     	    case CFGT_FLOAT:
-		printf ("%f\n", *((gfloat*)config->newvalue));
+		printf ("%f\n", *((gdouble*)config->newvalue));
 		break;
     	    case CFGT_STRING:
 		printf ("%s\n", config->newvalue ? config->newvalue : " ");
@@ -2031,7 +2077,7 @@ src_event_sink(SREvent *event_,
 }
 
 void
-brl_input_event (BrlInEvent *event)
+brl_input_event (BRLInEvent *event)
 {
     switch (event->event_type)
     {
@@ -2132,7 +2178,7 @@ void
 src_set_command_lines (const gchar *mouse_tracking,
 		       const gchar *focus_tracking,
 		       gboolean    invert,
-		       gint 	   zoomfactor)
+		       gdouble 	   zoomfactor)
 {
     if (mouse_tracking)
     {
@@ -2201,11 +2247,11 @@ src_set_command_lines (const gchar *mouse_tracking,
     if (0 < zoomfactor && 17 > zoomfactor)
     {
     	srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-			  CFGT_INT, 
+			  CFGT_FLOAT, 
 			  &zoomfactor,
 			  MAGNIFIER_CONFIG_PATH);
     	srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-			  CFGT_INT, 
+			  CFGT_FLOAT, 
 			  &zoomfactor,
 			  MAGNIFIER_CONFIG_PATH);
     }
